//
// modified of sample source Hadr00
// Ryuki Tanaka
//
#include "DetectorConstruction.hh"
#include "DetectorMessenger.hh"

#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"

#include "G4RunManager.hh"

#include "G4GeometryManager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SolidStore.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4UnitsTable.hh"
#include "G4ios.hh"
#include "G4ThreeVector.hh"

#include "G4NistManager.hh"

#include "HimeSD.hh"
#include "G4SDManager.hh"
#include "G4VSensitiveDetector.hh"

#include <sstream>
#include <algorithm>

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

G4int DetectorConstruction::moduleNumber = 1;
G4int DetectorConstruction::moduleMotherNumber = 1;

DetectorConstruction::DetectorConstruction()
  : moduleSize(100.0*cm, 4.0*cm, 2.0*cm), moduleGap(0.),
    moduleRotation(0), detectorPos(0., 0., 8.*m)
{
  logicModule = 0;
  logicModuleMother = 0;
  logicModuleGrandMother = 0;
  logicWorld  = 0;
  himeSD = 0;

  detectorMessenger = new DetectorMessenger(this);

  moduleMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_PLASTIC_SC_VINYLTOLUENE");
  moduleMotherMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_AIR");
  moduleGrandMotherMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_AIR");
  worldMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_AIR");
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

DetectorConstruction::~DetectorConstruction()
{
  delete detectorMessenger;
}

G4VPhysicalVolume* DetectorConstruction::Construct()
{
  // Cleanup old geometry
  G4GeometryManager::GetInstance()->OpenGeometry();
  G4PhysicalVolumeStore::GetInstance()->Clean();
  G4LogicalVolumeStore::GetInstance()->Clean();
  G4SolidStore::GetInstance()->Clean();

  // Module LV
  G4ThreeVector modulePos(0., -moduleSize.y()*moduleNumber/2.0 + moduleSize.y()/2.0, -moduleGap/2.0) ;
  G4Box* solidT = new G4Box("Module",
			    moduleSize.x()/2.,
			    moduleSize.y()/2.,
			    moduleSize.z()/2.);
  logicModule = new G4LogicalVolume( solidT,moduleMaterial,"Module");

  // Module Mother LV
  G4ThreeVector moduleMotherSize(moduleSize.x(),
				 moduleSize.y()*moduleNumber,
				 moduleSize.z()+moduleGap);
  G4ThreeVector moduleMotherPos(0., 0., -moduleMotherSize.z()*moduleMotherNumber/2.0 + moduleMotherSize.z()/2.0) ;
  G4Box* solidTM = new G4Box("ModuleMother",
			     moduleMotherSize.x()/2.,
			     moduleMotherSize.y()/2.,
			     moduleMotherSize.z()/2.);
  logicModuleMother = new G4LogicalVolume(solidTM,moduleMotherMaterial,"ModuleMother");

  // Module Grand-Mother LV
  G4double longSide = std::max(moduleMotherSize.x(), moduleMotherSize.y());
  G4ThreeVector moduleGrandMotherSize(longSide,
				       longSide, 
				       moduleMotherSize.z()*moduleMotherNumber);
  G4ThreeVector moduleGrandMotherPos(detectorPos.x()+0.1*mm,
				     detectorPos.y()+0.1*mm,
				     detectorPos.z()+moduleGrandMotherSize.z()/2.0) ;
  G4Box* solidTGM = new G4Box("ModuleGrandMother",
			      moduleGrandMotherSize.x()/2.,
			      moduleGrandMotherSize.y()/2.,
			      moduleGrandMotherSize.z()/2.);
  logicModuleGrandMother = new G4LogicalVolume(solidTGM,moduleGrandMotherMaterial,"ModuleGrandMother");

  // World LV
  G4ThreeVector worldSize(1.2*moduleGrandMotherSize.x()*2.,
			   1.2*moduleGrandMotherSize.y(),
			   1.2*2.*(moduleGrandMotherPos.z()+moduleGrandMotherSize.z()/2.0));
  G4Box* solidW = new G4Box("World",
			    worldSize.x()/2.,
			    worldSize.y()/2.,
			    worldSize.z()/2.) ;
  logicWorld = new G4LogicalVolume(solidW,worldMaterial,"World");
  
  // Module PV
  G4String moduleName("Module");
  G4ThreeVector modulePosShift(0., moduleSize.y(), 0.);
  for(G4int i = 0; i < moduleNumber; ++i){
    std::ostringstream os;
    os << moduleName << i;
    new G4PVPlacement(0,
		      modulePos,
		      logicModule,
		      os.str(),
		      logicModuleMother,
		      false,
		      i);
    modulePos += modulePosShift;
  }

  // Module Mother PV
  G4String moduleMotherName("ModuleMother");
  G4ThreeVector moduleMotherPosShift(0., 0., moduleMotherSize.z());
  for(G4int i = 0; i < moduleMotherNumber; ++i){
    G4RotationMatrix armRot= G4RotationMatrix();
    if(moduleRotation == 1){ 
      armRot.rotateZ(-90.*deg*(i%2));
    }
    std::ostringstream os;
    os << moduleMotherName << i;
    new G4PVPlacement(G4Transform3D(armRot, moduleMotherPos),
		      logicModuleMother,
		      os.str(),
		      logicModuleGrandMother,
		      false,
		      i);
    moduleMotherPos += moduleMotherPosShift;
  }

  // Module Grand-Mother PV
  new G4PVPlacement(0,
		    moduleGrandMotherPos,
		    logicModuleGrandMother,
		    "ModuleGrandMother",
		    logicWorld,
		    false,
		    0);

  // World PV
  G4ThreeVector worldPos(0., 0., 0.) ;  
  G4VPhysicalVolume* world = new G4PVPlacement(0,
					       worldPos,
					       logicWorld,
					       "World",
					       0,
					       false,
					       0);

  // add SD
  if(!himeSD){
    G4SDManager* SDman = G4SDManager::GetSDMpointer();
    himeSD = new HimeSD("/HimeSD");
    SDman->AddNewDetector(himeSD);
  }
  logicModule->SetSensitiveDetector(himeSD);

  // colors
  logicWorld->SetVisAttributes(G4VisAttributes::Invisible);
  logicModuleMother->SetVisAttributes(G4VisAttributes::Invisible);
  logicModuleGrandMother->SetVisAttributes(G4VisAttributes::Invisible);

  G4VisAttributes* regCcolor = new G4VisAttributes(G4Colour(0., 1.0, 1.0));
  logicModule->SetVisAttributes(regCcolor);

  // Print status
  G4cout << "### Module consist of " 
         << moduleMaterial->GetName() 
         <<  "  ###" << G4endl;

  G4cout << *(G4Material::GetMaterialTable()) << G4endl;

  return world;
}

void DetectorConstruction::SetModuleMaterial(const G4String& mat)
{
  // search the material by its name
  G4Material* material = G4NistManager::Instance()->FindOrBuildMaterial(mat);

  if (material && material != moduleMaterial) {
    moduleMaterial = material;
    if(logicModule) logicModule->SetMaterial(moduleMaterial);
    G4RunManager::GetRunManager()->PhysicsHasBeenModified();
  }
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void DetectorConstruction::SetWorldMaterial(const G4String& mat)
{
  // search the material by its name
  G4Material* material = G4NistManager::Instance()->FindOrBuildMaterial(mat);

  if (material && material != worldMaterial) {
    worldMaterial = material;
    if(logicWorld) logicWorld->SetMaterial(worldMaterial);
    G4RunManager::GetRunManager()->PhysicsHasBeenModified();
  }
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void DetectorConstruction::UpdateGeometry()
{
  G4RunManager::GetRunManager()->DefineWorldVolume(Construct());
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void DetectorConstruction::SetModuleNumber(G4int val)  
{
  if(val > 0 && val != moduleNumber){
    moduleNumber = val;
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetModuleMotherNumber(G4int val)  
{
  if(val > 0 && val != moduleMotherNumber){
    moduleMotherNumber = val;
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetModuleSizeX(G4double val)  
{
  if(val > 0.0 && val != moduleSize.x()){
    moduleSize.setX(val);
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetModuleSizeY(G4double val)  
{
  if(val > 0.0 && val != moduleSize.y()){
    moduleSize.setY(val);
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetModuleSizeZ(G4double val)  
{
  if(val > 0.0 && val != moduleSize.z()){
    moduleSize.setZ(val);
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetModuleGap(G4double val)  
{
  if(val >= 0.0 && val != moduleGap){
    moduleGap = val;
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetModuleRotation(G4int val)  
{
  if(val != moduleRotation){
    moduleRotation = val;
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetDetectorPosX(G4double val)  
{
  if(val != detectorPos.x()){
    detectorPos.setX(val);
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  }
}

void DetectorConstruction::SetDetectorPosY(G4double val)  
{
  if(val != detectorPos.y()){
    detectorPos.setY(val);
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  }
}

void DetectorConstruction::SetDetectorPosZ(G4double val)  
{
  if(val >= 0.0 && val != detectorPos.z()){
    detectorPos.setZ(val);
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  }
}

