//
#include "DetectorMessenger.hh"

#include "DetectorConstruction.hh"
#include "G4UIdirectory.hh"
#include "G4UIcmdWithABool.hh"
#include "G4UIcmdWithAString.hh"
#include "G4UIcmdWithAnInteger.hh"
#include "G4UIcmdWith3Vector.hh"
#include "G4UIcmdWithADoubleAndUnit.hh"
#include "G4UIcmdWithoutParameter.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

DetectorMessenger::DetectorMessenger(DetectorConstruction * Det)
:Detector(Det)
{
  testDir = new G4UIdirectory("/hime/");
  testDir->SetGuidance("Modification Commands for Hime");

  mattCmd = new G4UIcmdWithAString("/hime/ModuleMat",this);
  mattCmd->SetGuidance("Select Material for the Modules");
  mattCmd->SetParameterName("tMaterial",false);
  mattCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  matwCmd = new G4UIcmdWithAString("/hime/WorldMat",this);
  matwCmd->SetGuidance("Select Material for world");
  matwCmd->SetParameterName("wMaterial",false);
  matwCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  tnCmd = new G4UIcmdWithAnInteger("/hime/ModuleNumberOnOneLayer",this);
  tnCmd->SetGuidance("Set number of the Module on one layer");
  tnCmd->SetParameterName("moduleNumber",false);
  tnCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  tlnCmd = new G4UIcmdWithAnInteger("/hime/ModuleLayerNumber",this);
  tlnCmd->SetGuidance("Set number of the Layer");
  tlnCmd->SetParameterName("modulemotherNumber",false);
  tlnCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  txCmd = new G4UIcmdWithADoubleAndUnit("/hime/ModuleSizeX",this);
  txCmd->SetGuidance("Set X-Size of the module");
  txCmd->SetParameterName("moduleSizeX",false);
  txCmd->SetUnitCategory("Length");
  txCmd->SetRange("moduleSizeX>0");
  txCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  tyCmd = new G4UIcmdWithADoubleAndUnit("/hime/ModuleSizeY",this);
  tyCmd->SetGuidance("Set Y-Size of the module");
  tyCmd->SetParameterName("moduleSizeY",false);
  tyCmd->SetUnitCategory("Length");
  tyCmd->SetRange("moduleSizeY>0");
  tyCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  tzCmd = new G4UIcmdWithADoubleAndUnit("/hime/ModuleSizeZ",this);
  tzCmd->SetGuidance("Set Z-Size of the module");
  tzCmd->SetParameterName("moduleSizeZ",false);
  tzCmd->SetUnitCategory("Length");
  tzCmd->SetRange("moduleSizeZ>0");
  tzCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  tgapCmd = new G4UIcmdWithADoubleAndUnit("/hime/ModuleGap",this);
  tgapCmd->SetGuidance("Set Gap between the modules");
  tgapCmd->SetParameterName("moduleGap",false);
  tgapCmd->SetUnitCategory("Length");
  tgapCmd->SetRange("moduleGap>=0");
  tgapCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  trotCmd = new G4UIcmdWithAnInteger("/hime/ModuleRotation",this);
  trotCmd->SetGuidance("Set Rotation mode(1:rotate even modules pi/2, other:no rotation");
  trotCmd->SetParameterName("moduleRotation",false);
  trotCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  dxCmd = new G4UIcmdWithADoubleAndUnit("/hime/DetectorPosX",this);
  dxCmd->SetGuidance("Set X-Pos of the Detector");
  dxCmd->SetParameterName("detectorPosX",false);
  dxCmd->SetUnitCategory("Length");
  dxCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  dyCmd = new G4UIcmdWithADoubleAndUnit("/hime/DetectorPosY",this);
  dyCmd->SetGuidance("Set Y-Pos of the Detector");
  dyCmd->SetParameterName("detectorPosY",false);
  dyCmd->SetUnitCategory("Length");
  dyCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  dzCmd = new G4UIcmdWithADoubleAndUnit("/hime/DetectorPosZ",this);
  dzCmd->SetGuidance("Set Z-Pos of the Detector");
  dzCmd->SetParameterName("detectorPosZ",false);
  dzCmd->SetUnitCategory("Length");
  dzCmd->SetRange("detectorPosZ>=0");
  dzCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  updateCmd = new G4UIcmdWithoutParameter("/hime/update",this);
  updateCmd->SetGuidance("Update geometry.");
  updateCmd->SetGuidance("This command MUST be applied before \"beamOn\" ");
  updateCmd->SetGuidance("if you changed geometrical value(s)");
  updateCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

DetectorMessenger::~DetectorMessenger()
{
  delete testDir;
  delete mattCmd;
  delete matwCmd;
  delete tnCmd;
  delete tlnCmd;
  delete txCmd;
  delete tyCmd;
  delete tzCmd;
  delete tgapCmd;
  delete trotCmd;
  delete dxCmd;
  delete dyCmd;
  delete dzCmd;
  delete updateCmd;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void DetectorMessenger::SetNewValue(G4UIcommand* command, G4String newValue)
{
  if( command == mattCmd ) {
    Detector->SetModuleMaterial(newValue);
  } else if( command == matwCmd ) {
    Detector->SetWorldMaterial(newValue);
  } else if( command == tnCmd ) {
    Detector->SetModuleNumber(tnCmd->GetNewIntValue(newValue));
  } else if( command == tlnCmd ) {
    Detector->SetModuleMotherNumber(tlnCmd->GetNewIntValue(newValue));
  } else if( command == txCmd ) {
    Detector->SetModuleSizeX(txCmd->GetNewDoubleValue(newValue));
  } else if( command == tyCmd ) {
    Detector->SetModuleSizeY(tyCmd->GetNewDoubleValue(newValue));
  } else if( command == tzCmd ) {
    Detector->SetModuleSizeZ(tzCmd->GetNewDoubleValue(newValue));
  } else if( command == tgapCmd ) {
    Detector->SetModuleGap(tgapCmd->GetNewDoubleValue(newValue));
  } else if( command == trotCmd ) {
    Detector->SetModuleRotation(trotCmd->GetNewIntValue(newValue));
  } else if( command == dxCmd ) {
    Detector->SetDetectorPosX(dxCmd->GetNewDoubleValue(newValue));
  } else if( command == dyCmd ) {
    Detector->SetDetectorPosY(dyCmd->GetNewDoubleValue(newValue));
  } else if( command == dzCmd ) {
    Detector->SetDetectorPosZ(dzCmd->GetNewDoubleValue(newValue));
  } else if( command == updateCmd ) {
    Detector->UpdateGeometry();
  }
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

